require("helper")

local G = love.graphics



Model = Object:new()
function Model:init(filename)
	local scale = 50

	self.meshes = {}

	local ft = function(x) return tonumber(x) * scale end
	local fr = function(x) return tonumber(x) * math.pi / -180 end
	local fs = function(x) return math.abs(tonumber(x)) end
	local key_funcs = {ft, nil, ft, nil, fr, nil, fs, fs}

	local getLine = io.open(filename):lines()
	local line

	repeat line = getLine()
	until line == "Objects:  {"

	repeat
		line = getLine()
		local mesh_name = line:match('\tModel: "(.-)", "Mesh" {')
		if mesh_name then

			local mesh = { name = mesh_name, polys = {} }
			local poly = {}
			local verts = {}

			repeat
				line = getLine()
				local l = line:match("\t\tVertices: (.*)")
				if l then
					for x, y, _ in l:gfind("([^,]+),([^,]+),([^,]+)") do
						table.insert(verts, {
							tonumber(x) * scale, -tonumber(y) * scale
						})
					end
				end
				local l = line:match("\t\tPolygonVertexIndex: (.*)")
				if l then
					for i in l:gfind("[^,]+") do
						local e = false
						i = tonumber(i)
						if i < 0 then
							i = -i - 1
							e = true
						end
						table.insert(poly, verts[i + 1][1])
						table.insert(poly, verts[i + 1][2])
						if e then
							table.insert(mesh.polys, poly)
							poly = {}
						end
					end
				end
			until line == "\t}"


			self.meshes[mesh_name] = mesh
			table.insert(self.meshes, mesh)
		end

	until line == "}"



	-- read animation data
	repeat line = getLine()
	until line == "Takes:  {"

	repeat line = getLine()
	until line == '\tTake: "Default Take" {'

	repeat
		line = getLine()
		local mesh_name = line:match('\t\tModel: "(.-)" {')
		if mesh_name and self.meshes[mesh_name] then

			local anims = {}
			local chan = 0
			repeat
				line = getLine()

				if chan == 1 then
					local z = line:match('\t\t\t\t\t\tDefault: (.*)')
					if z then
						self.meshes[mesh_name].z = tonumber(z)
					end
				end

				if line:match('\t\t\t\t\t\tKey:') then
					chan = chan + 1

					if key_funcs[chan] then
						local anim = {}
						while true do
							line = getLine()
							local t, v = line:match("\t+(%d+),([^,]+)")
							if not t then break end
							local frame = tonumber(t) / 1924423250
							for f = #anim + 1, frame + 1 do
								anim[f] = key_funcs[chan](v)
							end

						end
						table.insert(anims, anim)
					end

				end
			until line == "\t\t}"
			self.meshes[mesh_name].anims = anims

		end
	until line == "\t}"

	table.sort(self.meshes, function(a, b) return a.z < b.z end)
end
function Model:draw()

	self.tick = (self.tick or 0) + 0.6


	for _, mesh in ipairs(self.meshes) do

		local q = function(i)
			local anim = mesh.anims[i]
			local t1 = math.floor(self.tick)
			local t2 = t1 + 1
			local t = self.tick - t1
			return anim[t1 % #anim + 1] * (1 - t) + anim[t2 % #anim + 1] * t
		end
		local p = {q(1), q(2), q(3), q(4), q(5)}


		G.setLineWidth(3)
		G.setColor(255, 255, 255)

		G.push()
		G.translate(p[1], p[2])
		G.rotate(p[3])
---[[
		for _, poly in ipairs(mesh.polys) do
			for i = 1, #poly, 2 do
				local j = (i + 2) % #poly
				G.line(	poly[i] * p[4], poly[i + 1] * p[5],
						poly[j] * p[4], poly[j + 1] * p[5])
			end
		end
--]]

		G.setColor(10, 10, 10)

		G.scale(p[4],p[5])
		for _, poly in ipairs(mesh.polys) do G.polygon("fill", poly) end

		G.pop()
	end
end



function love.keypressed(key)
	if key == "escape" then
		love.event.quit()
	end
end


function love.load()
	my_model = Model("run.fbx")
end


function love.update()
end


function love.draw()
	G.translate(400, 300)

	G.translate(0, 200)


	G.setColor(10, 10, 10)
	G.rectangle("fill", -400, 0, 800, 200)

	G.setLineWidth(1)
	G.setColor(255, 255, 255)
	G.line(-400, 0, 400, 0)

	my_model:draw()
end


